{-# LANGUAGE NamedFieldPuns #-}
{-# LANGUAGE FlexibleContexts #-}
module Text.TEIWA.Source.CSV (
      body
    , getContext
  ) where

import Control.Applicative ((<|>), many)
import Control.Monad.Except (MonadError(..))
import Data.Text.Lazy as Text (Text, pack, unpack)
import Text.Parsec (ParsecT, Stream, between, char, noneOf, sepBy, string, try)
import Text.TEIWA.Error (Error(..))
import Text.TEIWA.Source.Common (
    AnnotationContext(..), Field, Row, eol, recordLine
  )

field :: Stream s m Char => Char -> ParsecT s u m Field
field separator = Text.pack <$> (regular <|> quoted)
  where
    regular = many (noneOf $ separator:"\n\r\"")
    quoted = between quote quote $
      many (noneOf "\"" <|> try (string "\"\"" *> pure '"'))
    quote = char '"'

fields :: Stream s m Char => Char -> ParsecT s u m [Field]
fields separator = (field separator `sepBy` char separator) <* eol

body :: Stream s m Char => Char -> ParsecT s u m [Row]
body = many . recordLine . fields

getContext :: (Stream s m Char, MonadError Error m) =>
  Char -> Maybe Text -> ParsecT s u m AnnotationContext
getContext separator formColumn = fields separator >>= aux formColumn
  where
    aux Nothing [] = throwError EmptyHeader
    aux Nothing (columnName:header) =
      return $ AnnotationContext {columnIndex = 0, columnName, header}
    aux (Just columnName) headerRecord =
      case break (== columnName) headerRecord of
        (_, []) -> throwError . NoSuchColumn $ Text.unpack columnName
        (before, _:after) ->
          let columnIndex = length before in
          let header = (before ++ after) in
          return $ AnnotationContext {columnIndex, columnName, header}
