{-# LANGUAGE FlexibleContexts #-}
module Text.TEIWA.Source.Common (
      AnnotationContext(..)
    , Field
    , Row
    , eol
    , recordLine
  ) where

import Control.Applicative ((<|>))
import Data.Text.Lazy as Text (Text)
import Text.Parsec (
      Column, Line, ParsecT, Stream, char, endOfLine, getParserState, sourceLine, statePos
    , try
  )

data AnnotationContext = AnnotationContext {
      columnIndex :: Column
    , columnName :: Text
    , header :: Header
  }

type Field = Text
type Header = [Text]
type Row = (Line, [Field])

eol :: Stream s m Char => ParsecT s u m ()
eol = (try endOfLine <|> char '\r') *> return ()

recordLine :: Monad m => ParsecT s u m a -> ParsecT s u m (Line, a)
recordLine p = (,) <$> currentLine <*> p
  where
    currentLine = sourceLine . statePos <$> getParserState
