{-# LANGUAGE NamedFieldPuns #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE FlexibleContexts #-}
module Text.TEIWA.Source.ConLLX (
      getContext
    , sentences
  ) where

import Control.Applicative (many)
import Control.Monad.Except (MonadError(..))
import Data.Text.Lazy as Text (Text, pack, unpack)
import Text.Parsec (ParsecT, Stream, char, many1, noneOf, sepBy1)
import Text.TEIWA.Error (Error(..))
import Text.TEIWA.Source.Common (
    AnnotationContext(..), Field, Row, eol, recordLine
  )

fields :: [Field]
fields = [
      "ID"
    , "FORM"
    , "LEMMA"
    , "CPOSTAG"
    , "POSTAG"
    , "FEATS"
    , "HEAD"
    , "DEPREL"
    , "PHEAD"
    , "PDEPREL"
  ]

field :: Stream s m Char => ParsecT s u m Field
field = Text.pack <$> many1 (noneOf "\t\n\r")

row :: Stream s m Char => ParsecT s u m Row
row = recordLine (field `sepBy1` char '\t') <* eol

type Sentence = [Row]

sentence :: Stream s m Char => ParsecT s u m Sentence
sentence = many comment *> many1 row
  where
    comment = char '#' *> many (noneOf "\r\n") <* eol

sentences :: Stream s m Char => ParsecT s u m [Sentence]
sentences = many (sentence <* many1 eol)

getContext :: MonadError Error m => Maybe Text -> m AnnotationContext
getContext Nothing = return $ AnnotationContext {
      columnIndex = 1
    , columnName = "FORM"
    , header = take 1 fields ++ drop 2 fields
  }
getContext (Just columnName) =
  case break (== columnName) fields of
    (_, []) -> throwError . NoSuchColumn $ Text.unpack columnName
    (before, _:after) ->
      let columnIndex = length before in
      let header = (before ++ after) in
      return $ AnnotationContext {columnIndex, columnName, header}
